"""
pyrkube.client
~~~~~~~~~~~~~~

Utility classes and functions for pyrkube.

:copyright: (c) 2016 by Joe Black.
:license: Apache2.
"""

import json
import time
import logging
from pprint import pformat


class cached_property(object):
    '''Decorator for read-only properties evaluated only once within TTL period.

    It can be used to create a cached property like this::

        import random

        # the class containing the property must be a new-style class
        class MyClass(object):
            # create property whose value is cached for ten minutes
            @cached_property(ttl=600)
            def randint(self):
                # will only be evaluated every 10 min. at maximum.
                return random.randint(0, 100)

    The value is cached  in the '_cache' attribute of the object instance that
    has the property getter method wrapped by this decorator. The '_cache'
    attribute value is a dictionary which has a key for every property of the
    object which is wrapped by this decorator. Each entry in the cache is
    created only when the property is accessed for the first time and is a
    two-element tuple with the last computed property value and the last time
    it was updated in seconds since the epoch.

    The default time-to-live (TTL) is 300 seconds (5 minutes). Set the TTL to
    zero for the cached value to never expire.

    To expire a cached property value manually just do::

        del instance._cache[<property name>]

    '''
    def __init__(self, ttl=300):
        self.ttl = ttl

    def __call__(self, fget, doc=None):
        self.fget = fget
        self.__doc__ = doc or fget.__doc__
        self.__name__ = fget.__name__
        self.__module__ = fget.__module__
        return self

    def __get__(self, inst, owner):
        now = time.time()
        try:
            value, last_update = inst._cache[self.__name__]
            if self.ttl > 0 and now - last_update > self.ttl:
                raise AttributeError
        except (KeyError, AttributeError):
            value = self.fget(inst)
            if value:
                try:
                    cache = inst._cache
                except AttributeError:
                    cache = inst._cache = {}
                cache[self.__name__] = (value, now)
        return value


class ReprMixin:
    """Provide repr and IPython pretty print repr based on public attrs."""
    def __repr__(self):
        return '%s(%s)' % (
            type(self).__name__,
            pformat({k: v for k, v in self.items() if not k.startswith('_')})
        )

    def _repr_pretty_(self, p, cycle):
        if not cycle:
            pretty_repr(p, self)


class PrivAttrKeyMixin:
    """Add methods that provide dictionary type functions that exclude
    all but public attributes.
    """
    def keys(self):
        """dict like keys method for only public attributes."""
        return (k for k in dict.keys(self) if not k.startswith('_'))

    def values(self):
        """dict like values method for only public attributes."""
        return (v for k, v in dict.items(self) if not k.startswith('_'))

    def items(self):
        """dict like items method for only public attributes."""
        return ((k, v) for k, v in dict.items(self) if not k.startswith('_'))

    def __iter__(self):
        return iter(self.keys())

    def __len__(self):
        return len(list(self.keys()))


# class adict(ReprMixin, PrivAttrKeyMixin, attrdict.AttrDict):
#     """Customized attribute dictionary type."""
#     def __init__(self, *args, **kwargs):
#         def convert(o):
#             """Recursor for recursively converting object to adict."""
#             if isinstance(o, dict):
#                 return adict((k, convert(v)) for k, v in o.items())
#             elif isinstance(o, (list, tuple)):
#                 return type(o)(convert(v) for v in o)
#             return o

#         if args:
#             obj, *args = args
#         elif kwargs:
#             obj = kwargs
#         else:
#             obj = {}

#         if not isinstance(obj, types.GeneratorType):
#             obj = copy.deepcopy(obj)
#         attrdict.AttrDict.__init__(self, convert(obj), *args)

#     @classmethod
#     def _valid_name(cls, key):
#         return (
#             isinstance(key, six.string_types) and
#             re.match('^[A-Za-z_][A-Za-z0-9_]*$', key)
#         )
#         # and not hasattr(cls, key)

#     def __getattr__(self, key):
#         if key not in self or not self._valid_name(key):
#             raise AttributeError(
#                 "'{cls}' instance has no attribute '{name}'".format(
#                     cls=self.__class__.__name__, name=key
#                 )
#             )

#         return self[key]


class WrappedMapMixin:
    """Add dictionary accessor methods that wrap self._wrapped reference."""
    def __contains__(self, key):
        return key in self._wrapped

    def __getitem__(self, key):
        return self._wrapped[key]

    def __setitem__(self, key, value):
        self._wrapped[key] = value

    def __delitem__(self, key):
        del self._wrapped[key]

    def keys(self):
        """Return the wrapped object's keys method."""
        return self._wrapped.keys()

    def values(self):
        """Return the wrapped object's values method."""
        return self._wrapped.values()

    def items(self):
        """Return the wrapped object's items method."""
        return self._wrapped.items()

    def __iter__(self):
        return iter(self._wrapped)


def pretty_repr(p, obj):
    """Handles the ipython pretty printer repr object."""
    items = sorted(
        [(k, v) for k, v in obj.items() if not k.startswith('_')])

    with p.group(2, type(obj).__name__ + '(', ')'):
        p.breakable('')
        for index, (key, val) in enumerate(items):
            if index:
                p.text(",")
                p.breakable()
            with p.group(2, key + '=', ''):
                p.pretty(val)


def setup_logging(level):
    """This will setup logging for the module."""
    log = logging.getLogger('pyrkube')
    level = getattr(logging, level)
    log.setLevel(level)
    handler = logging.NullHandler()
    handler.setLevel(level)
    log.addHandler(handler)
    logging.raiseExceptions = False
    return log


def deserialize(value):
    """Safely attempt to deserialized a string."""
    try:
        return json.loads(value)
    except json.JSONDecodeError:
        return value



"""
pyrkube.hosts
~~~~~~~~~~~~~~

Kubernetes hostname classes for pyrkube.

:copyright: (c) 2016 by Joe Black.
:license: Apache2.
"""

import socket

from . import config


def lookup_addr(fqdn):
    """Lookup an IP Address by hostname."""
    socket.setdefaulttimeout()
    try:
        return socket.gethostbyname(fqdn)
    except socket.gaierror:
        pass


def get_hostname(obj, domain=None):
    """Return a Hostname object depending on the type of object passed."""
    domain = domain or config.DOMAIN

    if isinstance(obj, str):
        length = len(obj.split('.'))
        if length == 6:
            return StatefulPodHostname(obj, domain)
        elif length == 5:
            return PodHostname(obj, domain)
        else:
            raise RuntimeError('Not sure how to process host for %s', obj)
    else:
        owner = obj.owner
        if owner:
            if owner.kind == 'StatefulSet':
                return StatefulPodHostname(obj, domain)
        return PodHostname(obj, domain)


class PodHostnameBase:
    """Base class for PodHostnames."""
    def __init__(self, obj, domain='cluster.local'):
        self.domain = domain
        if isinstance(obj, str):
            self.fqdn = obj
        else:
            self._init_from_pod(obj)

    @property
    def fqdn(self):
        """Returns a calculated FQDN for the hostname."""
        parts = self._parts + ('namespace', 'domain')
        args = [getattr(self, p) for p in parts]
        return self._join_fqdn(*args)

    @fqdn.setter
    def fqdn(self, fqdn):
        """"Allows a new kube hostname object to be set by manipulating the
        FQDN.
        """
        self._init_from_fqdn(fqdn)

    def __repr__(self):
        return '%s(%r)' % (
            type(self).__name__,
            self.fqdn
        )

    def __str__(self):
        return self.fqdn

    def _join_fqdn(self, *args):
        return '.'.join(args[:-1] + (self.type, args[-1]))

    def __lt__(self, other):
        return self.hostname < other.hostname


class PodHostname(PodHostnameBase):
    """Represents a kubernetes 'Pod' hostname.

    A Kubernetes hostname where individual parts can be manipulated and the
    hostname is updated.  str(host) will return the hostname as a string.
    """

    type = 'pod'
    _parts = ('hostname',)

    def _init_from_pod(self, pod):
        self.ip = pod.status.podIP
        self.hostname = self.ip.replace('.', '-')
        self.namespace = pod.metadata.namespace

    def _init_from_fqdn(self, fqdn):
        self.hostname, self.namespace, _, self.domain = fqdn.split('.', 3)
        self.ip = self.hostname.replace('-', '.')

    def clone(self):
        """Clone's a copy of the current KubeHostname object.

        Options include requesting the KubeHostname for the master or by
        passing an index.
        """
        return type(self)(self.fqdn)


class StatefulPodHostname(PodHostnameBase):
    """Represents a kubernetes 'StatefulSet' hostname.

    A Kubernetes hostname where individual parts can be manipulated and the
    hostname is updated.  str(host) will return the hostname as a string.
    """

    type = 'svc'
    _parts = ('hostname', 'subdomain')

    @staticmethod
    def _get_hostname(pod):
        return (
            pod.metadata.get('hostname') or
            pod.metadata.annotations.get('pod.beta.kubernetes.io/hostname')
        )

    @staticmethod
    def _get_subdomain(pod):
        return (
            pod.metadata.get('subdomain') or
            pod.metadata.annotations.get('pod.beta.kubernetes.io/subdomain')
        )

    def _init_from_pod(self, pod):
        self.ip = pod.status.podIP
        self.hostname = self._get_hostname(pod)
        self.subdomain = self._get_subdomain(pod)
        self.namespace = pod.metadata.namespace

    def _init_from_fqdn(self, fqdn):
        # self.ip = lookup_addr(fqdn)

        self.hostname, self.subdomain, self.namespace, _, self.domain = (
            fqdn.split('.', 4)
        )

    @property
    def hostname(self):
        """Retrieves the first part of the kubernetes hostname.
        Example: `couchdb-0`
        """
        return '%s-%s' % (self.statefulset, self.index)

    @hostname.setter
    def hostname(self, hostname):
        """Allows the first part of the kubernetes hostname to be set and a
        new hostname object recalculated.
        """
        self.statefulset, index = hostname.split('-')
        self.index = int(index)

    @property
    def is_master(self):
        """Determines whether the index is 0 for when the master of a
        StatefulSet needs to be determined."""
        return self.index == 0

    def clone(self, master=False, index=None):
        """Clone's a copy of the current KubeHostname object.

        Options include requesting the KubeHostname for the master or by
        passing an index.
        """
        new = type(self)(self.fqdn)
        if index:
            new.index = index
        elif master:
            new.index = 0
        return new



import logging

import pyrkube
from pyrkube.objects import (
    Pod,
    ConfigMap,
    Secret,
    ReplicaSet,
    Deployment,
    StatefulSet,
    Endpoint,
    Service,
    KubeApp,
    KubeApps,
    Container,
    ContainerEnv,
    EndpointAddress,
    ResourceBase,
)
from pyrkube.adict import adict
from pyrkube.hosts import PodHostnameBase

from . import TestCase


class GenericObjectBase(TestCase):
    @classmethod
    def setUpClass(self):
        self.env = pyrkube.config.ENVIRONMENT
        self.api = pyrkube.client.KubeAPIClient(env=self.env)

    @classmethod
    def tearDownClass(self):
        self.env = None
        self.api = None


class ObjectBase:
    base_adicts = ('metadata',)

    def setUp(self):
        self.objs = self.api.get(self._class.__name__)

    def tearDown(self):
        self.objs = None

    def test_create_and_type(self):
        for obj in self.objs:
            self.assertIsNotNone(obj)
            self.assertTrue(len(repr(obj)))
            self.assertIsInstance(obj, self._class)

    def test_correct_kind(self):
        for obj in self.objs:
            self.assertEqual(obj.kind, self._class.kind)

    def test_api_object_added(self):
        for obj in self.objs:
            self.assertIn('_api', obj)
            self.assertIsInstance(obj._api, pyrkube.client.KubeAPIClient)

    def test_object_properties_exist_and_correct_type(self):
        for obj in self.objs:
            for attr in self.base_adicts:
                self.assertIn(attr, obj)
                self.assertIsInstance(getattr(obj, attr), adict)
            for attr in ('apiVersion', 'kind'):
                self.assertTrue(hasattr(obj, attr))
                self.assertIsInstance(getattr(obj, attr), str)

    def test_metadata_default_adicts_created(self):
        for obj in self.objs:
            for attr in ('labels', 'annotations'):
                self.assertIn(attr, obj.metadata)
                self.assertIsInstance(getattr(obj.metadata, attr), adict)

    def test_annotation_references_deserialized(self):
        for obj in self.objs:
            if obj.metadata.annotations.get('kubernetes.io/created-by'):
                self.assertIsInstance(
                    obj.metadata.annotations['kubernetes.io/created-by'], adict
                )
                ref = obj.metadata.annotations[
                    'kubernetes.io/created-by'].reference
                self.assertIsNotNone(ref)
                self.assertIsInstance(ref, adict)
                self.assertIn('kind', ref)
                self.assertIsInstance(ref['kind'], str)

    def test_name_property_exists_is_string_and_not_empty(self):
        for obj in self.objs:
            self.assertTrue(hasattr(obj, 'name'))
            self.assertIsInstance(obj.name, str)
            self.assertTrue(len(obj.name))

    def test_sort_on_name(self):
        objs = sorted(self.objs.copy())
        for i in range(len(objs) - 1):
            self.assertLess(objs[i], objs[i + 1])

    def test_ready_property_exists_and_is_boolean(self):
        if hasattr(self, 'has_ready') and self.has_ready:
            for obj in self.objs:
                self.assertTrue(hasattr(obj, 'ready'))
                self.assertIsInstance(obj.ready, bool)


class TestPodObject(ObjectBase, GenericObjectBase):
    base_adicts = ('spec', 'status')
    _class = Pod
    has_ready = True

    def setUp(self):
        self.base_adicts = self.base_adicts + ObjectBase.base_adicts
        ObjectBase.setUp(self)

    def tearDown(self):
        self.base_adicts = None
        ObjectBase.tearDown(self)

    @staticmethod
    def _get_containers(obj):
        if hasattr(obj.spec, 'containers'):
            return obj.spec.containers
        elif hasattr(obj.spec.template.spec, 'containers'):
            return obj.spec.template.spec.containers

    @staticmethod
    def _get_volumes(obj):
        if hasattr(obj.spec, 'volumes'):
            return obj.spec.volumes
        elif hasattr(obj.spec.template.spec, 'volumes'):
            return obj.spec.template.spec.volumes

    def test_containers_exist_and_are_adict(self):
        for obj in self.objs:
            containers = self._get_containers(obj)

            self.assertIsNotNone(containers)
            self.assertIsInstance(containers, adict)
            self.assertGreater(len(list(containers.values())), 0)

            for container in containers.values():
                self.assertIsNotNone(container)
                self.assertIsInstance(container, Container)
                for attr in ('env', 'image', 'name', 'resources'):
                    self.assertIn(attr, container)

    def test_environment_created(self):
        for obj in self.objs:
            for container in self._get_containers(obj).values():
                self.assertIn('env', container)
                self.assertIsNotNone(container.env)
                self.assertIsInstance(container.env, ContainerEnv)

    def test_environment_resolved(self):
        for obj in self.objs:
            for container in self._get_containers(obj).values():
                for key, value in container.env.items():
                    self.assertIsNotNone(key)
                    self.assertIsInstance(key, str)
                    self.assertGreater(len(key), 0)

                    self.assertIsNotNone(value)
                    self.assertIsInstance(value, str)
                    self.assertGreater(len(value), 0)

    def test_volumes(self):
        for obj in self.objs:
            volumes = self._get_volumes(obj)
            if volumes:
                self.assertIsInstance(volumes, list)
                for volume in volumes:
                    self.assertIn('name', volume)
                    self.assertIsInstance(volume.name, str)
                    self.assertGreater(len(volume.name), 0)


class TestDeploymentObject(TestPodObject):
    _class = Deployment

    def test_pods(self):
        for obj in self.objs:
            self.assertTrue(hasattr(obj, 'pods'))
            self.assertIsInstance(obj.pods, list)
            for pod in obj.pods:
                self.assertIsNotNone(pod)
                self.assertIsInstance(pod, Pod)


class TestReplicaSetObject(TestDeploymentObject):
    _class = ReplicaSet


class TestStatefulSetObject(TestDeploymentObject):
    _class = StatefulSet


class TestSecret(ObjectBase, GenericObjectBase):
    base_adicts = ('data',)
    _class = Secret
    has_ready = False

    def setUp(self):
        self.base_adicts = self.base_adicts + ObjectBase.base_adicts
        ObjectBase.setUp(self)

    def tearDown(self):
        self.base_adicts = None
        ObjectBase.tearDown(self)

    def test_data(self):
        for obj in self.objs:
            for key, val in obj.data.items():
                self.assertIsNotNone(key)
                self.assertIsInstance(key, str)
                self.assertGreater(len(key), 0)

                self.assertIsNotNone(key)
                self.assertIsInstance(key, str)
                self.assertGreater(len(key), 0)

    def test_custom_get(self):
        for obj in self.objs:
            for key in obj.data.keys():
                val = obj.get(key)
                self.assertIsNotNone(val)
                self.assertIsInstance(key, str)
                self.assertGreater(len(key), 0)


class TestConfigMap(TestSecret):
    _class = ConfigMap


class TestEndpoint(ObjectBase, GenericObjectBase):
    # base_adicts = ('subsets',)
    _class = Endpoint
    has_ready = True

    def setUp(self):
        # self.base_adicts = self.base_adicts + ObjectBase.base_adicts
        ObjectBase.setUp(self)
        self.objs = [obj for obj in self.objs if obj.name != 'kubernetes']

    def tearDown(self):
        self.base_adicts = None
        ObjectBase.tearDown(self)

    def test_endpoint_properties(self):
        for obj in self.objs:
            for attr in ('subsets', 'addresses'):
                val = getattr(obj, attr)
                self.assertIsNotNone(val)
                self.assertIsInstance(val, list)

    def test_addresses(self):
        for obj in self.objs:
            addresses = obj.addresses
            self.assertIsNotNone(addresses)
            self.assertIsInstance(addresses, list)

    def test_each_address_endpoint_correct_type(self):
        for obj in self.objs:
            for address in obj.addresses:
                self.assertIsNotNone(address)
                self.assertIsInstance(address, EndpointAddress)

    def test_each_address_endpoint_has_endpoint_reference(self):
        for obj in self.objs:
            for address in obj.addresses:
                self.assertIsNotNone(address._endpoint)
                self.assertIsInstance(address._endpoint, Endpoint)

    def test_each_address_endpoint_has_target_reference(self):
        for obj in self.objs:
            for address in obj.addresses:
                self.assertIsNotNone(address.targetRef)
                self.assertIsInstance(address.targetRef, adict)

    def test_address_endpoint_properties(self):
        for obj in self.objs:
            for address in obj.addresses:
                for attr in ('ip', 'nodeName'):
                    self.assertTrue(hasattr(address, attr))
                    val = getattr(address, attr)
                    self.assertIsNotNone(val)
                    self.assertIsInstance(val, str)

    def test_address_endpoint_targets(self):
        for obj in self.objs:
            for address in obj.addresses:
                target = address.target
                self.assertIsNotNone(target)
                self.assertIsInstance(target, Pod)

    def test_endpoint_pods(self):
        for obj in self.objs:
            pods = obj.pods
            for pod in pods:
                self.assertIsNotNone(pod)
                self.assertIsInstance(pod, Pod)
            for i in range(len(pods) - 1):
                self.assertLess(pods[i], pods[i + 1])

    def test_endpoint_nodes(self):
        for obj in self.objs:
            nodes = obj.nodes
            for node in nodes:
                self.assertIsNotNone(node)
                self.assertIsInstance(node, str)
                self.assertGreater(len(node), 0)

            # for i in range(len(nodes) - 1):
            #     self.assertLess(nodes[i], nodes[i + 1])

    def test_endpoint_ips(self):
        for obj in self.objs:
            ips = obj.ips
            for ip in ips:
                self.assertIsNotNone(ip)
                self.assertIsInstance(ip, str)
                self.assertGreater(len(ip), 0)

            for i in range(len(ips) - 1):
                self.assertLess(ips[i], ips[i + 1])

    def test_endpoint_hosts(self):
        for obj in self.objs:
            hosts = obj.hosts
            for host in hosts:
                self.assertIsNotNone(host)
                self.assertIsInstance(host, str)
                self.assertGreater(len(host), 0)

            for i in range(len(hosts) - 1):
                self.assertLess(hosts[i], hosts[i + 1])

    def test_endpoint_hostnames(self):
        for obj in self.objs:
            hostnames = obj.hostnames
            for hostname in hostnames:
                self.assertIsNotNone(hostname)
                self.assertIsInstance(hostname, PodHostnameBase)
                self.assertGreater(len(str(hostname)), 0)

            for i in range(len(hostnames) - 1):
                self.assertLess(hostnames[i], hostnames[i + 1])


class TestService(ObjectBase, GenericObjectBase):
    base_adicts = ('spec', 'status')
    _class = Service
    has_ready = False

    def test_has_endpoint(self):
        for obj in self.objs:
            ep = obj.endpoint
            self.assertIsNotNone(ep)
            self.assertIsInstance(ep, Endpoint)


class TestKubeApp(GenericObjectBase):
    def setUp(self):
        GenericObjectBase.setUp(self)
        self.objs = [
            KubeApp(self.api, name) for name in ('couchdb', 'rabbitmq')]

    def tearDown(self):
        GenericObjectBase.tearDown(self)
        self.objs = None

    def test_created_and_correct_type(self):
        for obj in self.objs:
            self.assertIsNotNone(obj)
            self.assertIsInstance(obj, KubeApp)
            self.assertGreater(len(repr(obj)), 0)

    def has_correct_api_object(self):
        for obj in self.objs:
            api = getattr(obj, 'api')
            self.assertTrue(api)
            self.assertIsInstance(obj._api, pyrkube.client.KubeAPIClient)

    def test_object_attributes_set_correct_type_and_value(self):
        for obj in self.objs:
            self.assertTrue(hasattr(obj, 'name'))
            self.assertIsInstance(obj.name, str)
            self.assertGreater(len(obj.name), 0)

            self.assertTrue(hasattr(obj, 'sleep'))
            self.assertIsInstance(obj.sleep, int)
            self.assertGreater(obj.sleep, 0)

    def test_kind(self):
        for obj in self.objs:
            self.assertTrue(obj.kind)
            self.assertIsNotNone(obj.kind)
            self.assertIsInstance(obj.kind, str)
            self.assertIn(obj.kind, ('Deployment', 'StatefulSet'))

    def test_resource(self):
        for obj in self.objs:
            self.assertTrue(obj.resource)
            self.assertIsNotNone(obj.resource)
            self.assertIsInstance(obj.resource, ResourceBase)
            self.assertIn(type(obj.resource).__name__,
                          ('Deployment', 'StatefulSet'))

    def test_loaded(self):
        for obj in self.objs:
            self.assertIsNotNone(obj.loaded)
            self.assertIsInstance(obj.loaded, bool)

    def test_ready(self):
        for obj in self.objs:
            self.assertIsNotNone(obj.ready)
            self.assertIsInstance(obj.ready, bool)


class TestKubeApps(GenericObjectBase):
    def setUp(self):
        GenericObjectBase.setUp(self)
        self.names = ('couchdb', 'rabbitmq')
        self.obj = KubeApps(self.api, self.names, logger='tests')

    def tearDown(self):
        GenericObjectBase.tearDown(self)
        self.names = None
        self.obj = None

    def test_api_is_set_and_correct_class(self):
        self.assertTrue(hasattr(self.obj, 'api'))
        self.assertIsInstance(self.obj.api, pyrkube.client.KubeAPIClient)

    def test_attributes_set_and_correct_type(self):
        self.assertTrue(hasattr(self.obj, 'names'))
        self.assertIsInstance(self.obj.names, tuple)

        self.assertTrue(hasattr(self.obj, 'logging'))
        self.assertIsInstance(self.obj.logging, bool)

        if self.obj.logging:
            self.assertIsInstance(self.obj.logger, logging.Logger)

        self.assertTrue(hasattr(self.obj, 'sleep'))
        self.assertTrue(self.obj.sleep, int)

    def test_get_app_returns_app(self):
        app = self.obj._get_app('rabbitmq')
        self.assertIsNotNone(app)
        self.assertIsInstance(app, KubeApp)
        self.assertEqual(app.name, 'rabbitmq')

    def test_apps_loaded(self):
        self.assertIsNotNone(self.obj.apps)
        self.assertIsInstance(self.obj.apps, dict)
        self.assertIs(self.obj.apps, self.obj._wrapped)

    def test_ready_is_correct(self):
        self.assertTrue(hasattr(self.obj, 'ready'))
        self.assertIsInstance(self.obj.ready, bool)

    def test_status(self):
        self.assertIsInstance(self.obj.status, dict)
        self.assertListEqual(
            sorted(self.obj.status.keys()),
            sorted(self.obj.names)
        )
        for key, val in self.obj.status.items():
            self.assertIsNotNone(key)
            self.assertIsInstance(key, str)

            self.assertIsNotNone(val)
            self.assertIsInstance(val, bool)

    def test_wait_emits_logs(self):
        with self.assertLogs('tests', level='INFO') as cm:
            self.obj.wait()
            self.assertGreater(len(cm.output), 0)



"""
PyRKube
~~~~~~~~~~~~~

Readonly wrapper for pykube with a different focus.
"""

import logging

from . import config, util, adict, client, hosts, objects
from .client import KubeAPIClient, KubeInterfaceBase
from .hosts import PodHostname, StatefulPodHostname
from .objects import (
    Pod,
    ConfigMap,
    Secret,
    ReplicaSet,
    Deployment,
    DaemonSet,
    StatefulSet,
    Endpoint,
    Service,
    KubeApp,
    KubeApps
)

__title__ = 'pyrkube'
__version__ = '0.2.0'
__build__ = 0x000200
__author__ = "Joe Black <joe@valuphone.com>"
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2016 Joe Black'


logging.getLogger(__name__).addHandler(logging.NullHandler())



import re
from setuptools import setup, find_packages


with open('pyrkube/__init__.py', 'rt') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.rst') as fd:
    long_description = fd.read()


setup(
    name='pyrkube',
    version=version,
    description='Readonly wrapper for pykube with a different focus',
    long_description=long_description,
    keywords=['kubernetes', 'clustering', 'templating'],
    author='Joe Black',
    author_email='joeblack949@gmail.com',
    url='https://github.com/joeblackwaslike/pyrkube',
    download_url='https://github.com/joeblackwaslike/pyrkube/tarball/0.2.0',
    license='Apache 2.0',
    zip_safe=False,
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    install_requires=[
        'pykube',
        'six'
    ],
    # setup_requires=[
    #     'pytest-runner'
    # ],
    tests_require=['pytest'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        "License :: OSI Approved :: Apache Software License",
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: System :: Clustering',
    ]
)
